#!/usr/bin/python3

"""Utility to retrieve information about WMI devices on Linux """

__version__ = "1.1.0"
__author__ = "Armin Wolf"
__email__ = "W_Armin@gmx.de"
__all__ = (
    "device",
    "provider",
    "main"
)
